@@ -30,26 +30,6 @@ round_any <- function(x, accuracy)
   roundc(x / accuracy) * accuracy
 }
 
-make_grid = function(xmin, xmax, ymin, ymax, res, start = c(0,0))
-{
-  xo = seq(f_grid(xmin, res, start[1]), f_grid(xmax, res, start[1]), res)
-  yo = seq(f_grid(ymin, res, start[2]), f_grid(ymax, res, start[2]), res)
-
-  grid = expand.grid(X = xo, Y = yo)
-  data.table::setDT(grid)
-
-  return(grid)
-}
-
-
-group_grid = function(x, y, res, start = c(0,0))
-{
-  xgrid = f_grid(x, res, start[1])
-  ygrid = f_grid(y, res, start[2])
-
-  return(list(Xgrid = xgrid, Ygrid = ygrid))
-}
-
 group_grid_3d = function(x, y, z, res, start = c(0,0,0))
 {
   xgrid = f_grid(x, res[1], start[1])
@@ -70,23 +50,28 @@ verbose = function(...)
     cat(..., "\n")
 }
 
-dummy_las = function(n, seeds = 1)
-{
+dummy_las <- function(n, seeds = 1) {
   set.seed(seeds)
-  X = stats::runif(n, 0, 100)
+  X <- round_any(stats::runif(n, 0, 100), 0.001)
   set.seed(seeds + 1)
-  Y = stats::runif(n, 0, 100)
+  Y <- round_any(stats::runif(n, 0, 100), 0.001)
   set.seed(seeds + 2)
-  Z = c(stats::runif(0.8*n, 0, 25), rep(0, 0.2*n))
-  Classification = as.integer(c(rep(1, 0.8*n), rep(2, 0.2*n)))
+  Z <- round_any(c(stats::runif(0.8 * n, 0, 25), rep(0, 0.2 * n)), 0.001)
+  Classification <- as.integer(c(rep(1, 0.8 * n), rep(2, 0.2 * n)))
   set.seed(seeds + 3)
-  Intensity = as.integer(stats::runif(n, 10, 50))
-  ReturnNumber    = as.integer(rep(c(1,1,1,2,3,1,2,1,2,1), n/10))
-  NumberOfReturns = as.integer(rep(c(1,1,3,3,3,2,2,2,2,1), n/10 ))
-
-  dt = data.table::data.table(X, Y, Z, Classification, Intensity, ReturnNumber, NumberOfReturns)
-  las = suppressWarnings(LAS(dt, check = FALSE))
+  Intensity <- as.integer(stats::runif(n, 10, 50))
+  ReturnNumber <- as.integer(rep(c(1, 1, 1, 2, 3, 1, 2, 1, 2, 1), n / 10))
+  NumberOfReturns <- as.integer(rep(c(1, 1, 3, 3, 3, 2, 2, 2, 2, 1), n / 10))
 
+  data <- data.table::data.table(X, Y, Z, Classification, Intensity, ReturnNumber, NumberOfReturns)
+  header <- rlas::header_create(data)
+  header[["X scale factor"]] <- 0.001
+  header[["Y scale factor"]] <- 0.001
+  header[["Z scale factor"]] <- 0.001
+  header[["X offset"]] <- 0
+  header[["Y offset"]] <- 0
+  header[["Z offset"]] <- 0
+  las <- suppressMessages(suppressWarnings(LAS(data, header, check = FALSE)))
   return(las)
 }
 
